
import os, sys
from classes.add_instruction import AddInstruction
from classes.run_instructions import RunInstructions

class Bootstrap:
    def __init__(self):
        self.interactive_retry = 0
        
        self.instructions = {
            "1": "move",
            "2": "copy",
            "3": "replace",
            "4": "minify",
            "5": "command"
        }
        
        
    def list_instructions(self):
        ins_keys = self.instructions.keys()
        
        for key in ins_keys:
            print("{0}. {1}".format(key , self.instructions[key]))
        
    def get_directory(self, directory):
        if (directory == None):
            target_dir = os.getcwd()
        else:
            target_dir = os.path.abspath(directory)
        
        return target_dir
    
    
    def generate(self, directory):
        print("[*] Generating config file")
        
        target_dir = self.get_directory(directory)
            
        with open(os.path.join(target_dir, "optimizo.json"), "w") as wobj:
            wobj.write("""{}""")
        
        print("[100%] Done")
        
        
    def add_instruction_semi_interactive(self, group, instruction, directory):
        target_dir = self.get_directory(directory)
        add_instruction = AddInstruction(group, instruction, target_dir)
        
    def add_instruction_interactive(self, group, directory):
        
        target_dir = self.get_directory(directory)
        
        
        self.list_instructions()
        
        ins = input("Choose instruction[1-5]: ")
        
  
        try:
            ins = int(ins)
            choosen_ins = self.instructions[str(ins)]
            add_instruction = AddInstruction(group, choosen_ins, target_dir)
            
             
        except ValueError:
            
            if (self.interactive_retry < 2):
                self.interactive_retry += 1
                print("[-] Please a number between 1-5")
                self.add_instruction_interactive() 
            else:
                print("[-] You dont know what you are doing! Exiting...")
                sys.exit()
        
    def run_instruction(self, group, directory):
       target_dir = self.get_directory(directory) 
       run_inst = RunInstructions(group, target_dir)
        


import css_html_js_minify as minify_res

class Minify:
    def __init__(self):
        pass
    
    def html_file(self, file_name, overwrite=False):
        minify_res.process_single_html_file(file_name, overwrite)
    
    def js_file(self, file_name, overwrite=False):
        minify_res.process_single_js_file(file_name, overwrite)
    
    def css_file(self, file_name, overwrite=False):
        minify_res.process_single_css_file(file_name, overwrite)
    
    def html_str(self, string):
        return minify_res.html_minify(string)
    
    def js_str(self, string):
        return minify_res.js_minify(string)
    
    def css_str(self, string):
        return minify_res.css_minify(string,  comments=False)
    
    









import os
import json
import shutil
import re
import subprocess
from classes.minify import Minify

class RunInstructions:
    def __init__(self, group, directory):
        self.group = group
        self.directory = directory
        self.filename = os.path.join(self.directory, "optimizo.json")
        
        self.fetch_instructions()
   

    def fetch_instructions(self):
        with open(self.filename) as readobj:
            try:
                data = json.load(readobj)
                
                if self.group in data.keys():
                    instructions = data[self.group]
                    self.run_instructions(instructions)
                else:
                    print("[-] Group doesnt exist...")
            except json.JSONDecodeError:
                print("[-] Please try adding an instruction")
            
    def run_instructions(self, instructions):
        for ins in instructions:
            ins_name = ins["name"]
            
            if ins_name == "move":
                self.move_ins(ins)
            elif ins_name == "copy":
                self.copy_ins(ins)
            elif ins_name == "replace":
                self.replace_ins(ins)
            elif ins_name == "minify":
                self.minify_ins(ins)
            elif ins_name == "command":
                self.command_ins(ins)
            
        print("[+] Done executing instructions")
                    

    def move_ins(self,obj):
        src_file = obj["src"]
        dst_path = obj["dst"]
        
        print("Moving {0} to {1}".format(src_file, dst_path))
        shutil.move(src_file, dst_path)
    
    def copy_ins(self,obj):
        src_file = obj["src"]
        dst_path = obj["dst"]
        
        print("Copy {0} to {1}".format(src_file, dst_path))
        shutil.copy(src_file, dst_path)
        
    
    def replace_ins(self, obj):
        file_name = obj["src"]
        search_str = obj["search"]
        repl_str = obj["replace"]
        
        print("[+] Replacing {0} with {1}".format(search_str, repl_str))
        
        with open(file_name, "r+") as f:
            text = f.read()
            text = re.sub(search_str, repl_str, text)
            f.seek(0)
            f.write(text)
            f.truncate()
            
    def minify_ins(self, obj):
        source_file = obj["src"]
        print("[+] Minifying {0}".format(source_file))
        
        minify = Minify()
        
        if (source_file.endswith("css")):
            minify.css_file(source_file)
        elif (source_file.endswith("html") or source_file.endswith("blade.php")):
            minify.html_file(source_file, True)
        elif (source_file.endswith("js")):
            minify.js_file(source_file)
        else:
            print("[-] File extension not supported")
        
        
        
    
    def command_ins(self, obj):
        command = obj['command']
        print("[+] Running command {0}".format(command))
        subprocess.run(command) 
    
    


"""Optimize Grapper"""
import argparse

import os
import sys


from classes.bootstrap import Bootstrap


class Optimizo:
    def __init__(self):
        self.process_commands()  
        

    def process_commands(self):
        # Create the parser
        my_parser = argparse.ArgumentParser(prog="optimizo", 
                                            description='Optimizo its an tool that allows to set up instructions and run them later. In the mean you will have to run them manually',
                                            epilog='Happy Automating and Optimizing!')

        # Add the arguments
        my_parser.add_argument('-c',
                            '--generate',
                            action='store_true',
                            help='Generate optimizo config file. If you dont specify directory(using -w) it will generate in the current directory.')


        my_parser.add_argument('-g',
                            '--group',
                            metavar='group',
                            type=str,
                            help='Group to work with. If group doesnt exist it will be created.')
        
        my_parser.add_argument('-l',
                            '--list',
                            action='store_true',
                            help='List supported instructions.')
        
        my_parser.add_argument('-i',
                            '--instruction',
                            type=str,
                            metavar='instruction',
                            help='Specify an instruction. For use in semi-interactive')

        my_parser.add_argument('-r',
                            '--run',
                            metavar='run',
                            type=str,
                            help='Run instructions under a specific group.')

        my_parser.add_argument('-a',
                            '--add',
                            action='store_true',
                            help='Add an instruction')


        my_parser.add_argument('-w',
                            '--directory',
                            action='store',
                            help='Working directory. Directory with your project')


        # Execute the parse_args() method
        args = my_parser.parse_args()
        bootstrap = Bootstrap() 


        try:
            if (args.list):
                print("\nSupported Instructions:")
                bootstrap.list_instructions()
                
            if (args.generate):
                bootstrap.generate(args.directory)
            
            if (args.group and args.add):
                bootstrap.add_instruction_interactive(args.group, args.directory)
                
            if (args.group and args.instruction):
                bootstrap.add_instruction_semi_interactive(args.group, args.instruction, args.directory)
                
            if (args.run):
                bootstrap.run_instruction(args.run, args.directory)

        except KeyboardInterrupt:
            print("\n\n[0] Goodbye, Exiting!")
    



if __name__ == '__main__':
    optimizo = Optimizo()


from setuptools import setup

with open("README.md", 'r') as fileobj:
    long_description = fileobj.read()

setup(
   name='optimizo',
   version='1.4.0',
   description='Optimizo its an tool that allows to set up instructions and run them later. In the mean you will have to run them manually',
   license="MIT",
   long_description=long_description,
   long_description_content_type="text/markdown",
   author='Takunda Madechangu',
   author_email='madechangu.takunda@gmail.com',
   url="http://taku.co.zw",
   packages=['classes', '.'],  #same as name
   install_requires=[
       'appdirs==1.4.4',
        'autopep8==1.5.7',
        'css-html-js-minify==2.5.5',
        'distlib==0.3.2',
        'filelock==3.0.12',
        'pycodestyle==2.7.0',
        'six==1.16.0',
        'tinydb==4.5.0',
        'toml==0.10.2',
        'virtualenv==20.4.7',
        'virtualenvwrapper-win==1.2.6',
   ], #external packages as dependencies
   
)


import os
import json
import classes.bootstrap as appbootstrap

class AddInstruction:
    def __init__(self,group, name, target_dir):
        self.group = group
        self.name = name
        self.target_dir = target_dir
        
        self.retry_adding = 0
    
        
        if (self.name == "move"):
            self.get_move()
        elif (self.name == "copy"):
            self.get_copy()
        elif (self.name == "replace"):
            self.get_replace()
        elif (self.name == "minify"):
            self.get_minify()
        elif (self.name == "command"):
            self.get_command()
        else:
            print("[-] Instruction not supported.")

        
        
    def add_instruction(self, instruction_object):
        filepath = os.path.join(self.target_dir, "optimizo.json")
        
        
        try:
            with open(filepath) as obj:
                try:
                    data = json.load(obj)
                    
                    if self.group in data.keys():
                        data[self.group].append(instruction_object)
                    else:
                        data[self.group] = []
                        data[self.group].append(instruction_object)
                        
                    with open(filepath, "w") as outfile:
                        json.dump(data, outfile)
                except json.decoder.JSONDecodeError:
                    with open(filepath, "w") as outfile:
                        data = {}
                        
                        with open(filepath, "w") as outfile:
                            json.dump(data, outfile)
                            
                    if self.retry_adding < 2:
                        self.retry_adding += 1
                        self.add_instruction(instruction_object)
                    else:
                        print("[-] Couldnt add instruction, exiting")
                            
            print("[+] Added")
        except FileNotFoundError:
            print("[-] File not found adding file...")
            bootstrap = appbootstrap.Bootstrap()
            bootstrap.generate(self.target_dir)
            
            
            if self.retry_adding < 2:
                self.retry_adding += 1
                self.add_instruction(instruction_object)
            else:
                print("[-] Couldnt add instruction, exiting")
                    
        
        
    def get_move(self):
        print("\nMOVE INSTRUCTION:\n")
        source_file = input("Source File: ")
        dest_file = input("Dest Directory: ")
        
        if (os.path.exists(os.path.abspath(source_file))):
            if os.path.exists(os.path.abspath(dest_file)):
                if (os.path.isdir(os.path.abspath(dest_file))):
                    self.add_instruction(
                        {
                            "name": self.name,
                            "src": os.path.abspath(source_file),
                            "dst": os.path.abspath(dest_file)
                        }
                    )
                else:
                    print("[-] Destination file is not a directory, exiting")
            else:
                print("[-] Destination directory doesnt exist, exiting...")
        else:
            print("[-] Source file doesnt exist, exiting...")
            
    def get_copy(self):
        print("\nCOPY INSTRUCTION:\n")
        source_file = input("Source File: ")
        dest_file = input("Dest Directory: ")
        
        if (os.path.exists(os.path.abspath(source_file))):
            if os.path.exists(os.path.abspath(dest_file)):
                if (os.path.isdir(os.path.abspath(dest_file))):
                    self.add_instruction(
                        {
                            "name": self.name,
                            "src": os.path.abspath(source_file),
                            "dst": os.path.abspath(dest_file)
                        }
                    )
                else:
                    print("[-] Destination file is not a directory, exiting")
            else:
                print("[-] Destination directory doesnt exist, exiting...")
        else:
            print("[-] Source file doesnt exist, exiting...")
        
    def get_replace(self):
        print("\nREPLACE INSTRUCTION:\n")
        source_file = input("Source File: ")
        search_str = input("Search String: ")
        repl_str = input("Replace String: ")
        
        file_name = os.path.abspath(source_file)
        
        if os.path.exists(file_name):
            if (os.path.isfile(file_name)):
                self.add_instruction({
                    "name": self.name,
                    "src": file_name,
                    "search": search_str,
                    "replace": repl_str
                })
            else:
                print("[-] File must be an actual file XD")
        else:
            print("[-] File doesnt exist")
            
    def get_minify(self):
        print("\nMINIFY INSTRUCTION:\n")
        source_file = input("Source File: ")
        
        file_name = os.path.abspath(source_file)
        
        if os.path.exists(file_name):
            if (os.path.isfile(file_name)):
                self.add_instruction({
                    "name": self.name,
                    "src": file_name,
                
                })
            else:
                print("[-] File must be an actual file XD")
        else:
            print("[-] File doesnt exist")
            
    def get_command(self):
        print("\nCOMMAND INSTRUCTION:\n")
        command = input("Command: ")

        self.add_instruction({
                    "name": self.name,
                    "command": command
                
        })
        
        
        
        
    